/* callbacks.c - controls GTK callback functions (excluding network) */

#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>

#include <gnome.h>

#include <ctype.h>
#include <unistd.h>

#include "napster.h"
#include "gnapster.h"
#include "callbacks.h"
#include "ui.h"
#include "upload.h"
#include "network.h"
#include "commands.h"

#include "config.h"

extern CONN_INFO conn_info;
extern USER_INFO user_info;
extern GNAPSTER_MAIN gnapster_main;
extern SEARCH_TAB search_tab;
extern BROWSE_TAB browse_tab;
extern DOWNLOAD_TAB download_tab;
extern UPLOAD_TAB upload_tab;
extern CONSOLE_TAB console_tab;
extern CDDB_TAB cddb_tab;
extern MOTD_TAB motd_tab;
extern PROP_DLG prop_dlg;
extern BUILD_DLG build_dlg;
extern BROWSE_DLG browse_dlg;
extern ADD_DLG add_dlg;
extern EDIT_DLG edit_dlg;
extern LIST_CHAN_DLG list_chan_dlg;
extern RESUME_DLG resume_dlg;

extern int building;
extern GList *total_list;
extern GList *ignore_list;

#define MAX_HISTORY 15

#define DOWNLOAD_FORCE 1 << 0
#define DOWNLOAD_NORMAL 1 << 1

int loc_curr = 0, loc_end = 0;
GList *console_history = NULL;

void gnapster_exit_cb() {
   if (conn_info.connected) {
      close(conn_info.sock); 
      gdk_input_remove(conn_info.sock_input);
   }
   gtk_main_quit();
}

int remote_queue_check(gpointer data) {
   NAP_DOWNLOAD *download;
   
   if (!data)
     return 0;
   
   download = (NAP_DOWNLOAD *)data;
   
   napster_send(conn_info.sock, NAPSTER_GET, "%s \"%s\"",
		download->user, download->file);
   
   return 0;
}

void queue_check() {
   int downloads, queue;
   NAP_DOWNLOAD *q, *dup;

   queue = g_list_length(GTK_CLIST(download_tab.queue_clist)->row_list);
   if (!queue)
     return;

   downloads = g_list_length(GTK_CLIST(download_tab.clist)->row_list);
   if (downloads >= user_info.conf[MAX_DOWNLOADS])
     return;
   
   q = gtk_clist_get_row_data(GTK_CLIST(download_tab.queue_clist), 0);
   if (!q)
     return;
 
   dup = download_dup(q);
   handle_download(dup, DOWNLOAD_NORMAL);
   
   gtk_clist_remove(GTK_CLIST(download_tab.queue_clist), 0);   
}

void add_queue(NAP_DOWNLOAD *data) {
   if (!data)
     return;
   
   gnapster_clist_append(download_tab.queue_clist, download_tab.vbox, data,
			 data->trunc_file, data->user, NULL);
}

void handle_download(NAP_DOWNLOAD *data, unsigned int flags) {
   int downloads;
   
   if (!data)
     return;
   
   downloads = g_list_length(GTK_CLIST(download_tab.clist)->row_list);
   
   if (!(flags & DOWNLOAD_FORCE) &&
       user_info.conf[MAX_DOWNLOADS_SWITCH] &&
       downloads >= user_info.conf[MAX_DOWNLOADS]) {
      add_queue(data);
      return;
   }
   
   network_add_download(data);

   napster_send(conn_info.sock, NAPSTER_GET, "%s \"%s\"",
		data->user, data->file);
}

void resume_dlg_clist_cb() {
}

void resume_dlg_cb(GtkWidget *w, gpointer data) {
   char *s;
   
   if (!data) 
     return;
   
   s = (char *)data;
   if (!strcmp(s, "resume")) {
      NAP_RESUME *temp;
      
      temp = gnapster_get_selection_data(resume_dlg.clist, NULL);
      if (!temp)
	return;
      napster_send(conn_info.sock, NAPSTER_RESUME_SEARCH, "%s-%i %i",
		   temp->checksum, temp->remotesize, temp->remotesize);
      conn_info.searching = 1;
      gtk_clist_clear(GTK_CLIST(search_tab.clist));
      gtk_clist_freeze(GTK_CLIST(search_tab.clist));
      gtk_entry_set_editable(GTK_ENTRY(search_tab.gtk_entry), 0);
   } else if (!strcmp(s, "remove")) {
      NAP_RESUME *temp;
      NAP_DOWNLOAD *download;
//      int row = -1;
      
      temp = gnapster_get_selection_data(resume_dlg.clist, NULL);
      if (!temp)
	return;
      download = g_malloc(sizeof(NAP_DOWNLOAD));
      memset(download, 0, sizeof(NAP_DOWNLOAD));
      download->type = N_DOWNLOAD;
      download->id = g_strdup_printf("%s-%i", temp->checksum, 
				     temp->remotesize);
      remove_resume_entry(download);
      g_free(download);
//      gtk_clist_remove(GTK_CLIST(resume_dlg.clist), row);
      gtk_clist_clear(GTK_CLIST(resume_dlg.clist));
      gtk_clist_freeze(GTK_CLIST(resume_dlg.clist));
      populate_resume_dlg();
      gtk_clist_thaw(GTK_CLIST(resume_dlg.clist));
      return;
   }
   
   gnome_dialog_close(GNOME_DIALOG(resume_dlg.window));
   resume_dlg.window = NULL;
}

void populate_resume_dlg() {
   FILE *f;
   char *conf, *ptr, buf[1024 + 1], *tf;
   char *file, *local, *remote;
   char lstr[26], rstr[26];
   NAP_RESUME *resume;
   
   conf = g_strdup_printf("%s/.gnapster/resume", getenv("HOME"));
   f = fopen(conf, "r");
   g_free(conf);
   if (!f)
     return;
   while(!feof(f)) {
      fgets(buf, 1024, f);
      if (feof(f)) break;
      ptr = buf;
      file = next_arg(ptr, &ptr);
      local = next_arg(ptr, &ptr);
      remote = next_arg(ptr, &ptr);
      if (!remote)
	break;
      resume = g_malloc(sizeof(NAP_RESUME));
      memset(resume, 0, sizeof(NAP_RESUME));
      resume->type = N_RESUME;
      resume->file = strdup(file);
      resume->localsize = my_atoi(local);
      resume->checksum = strdup(next_arg_full(remote, &remote, '-'));
      resume->remotesize = my_atoi(remote);
      tf = strrchr(resume->file, '/');
      if (tf)
	tf++;
      strcpy(lstr, simplify_size(resume->localsize));
      strcpy(rstr, simplify_size(resume->remotesize));
      gnapster_clist_append(resume_dlg.clist, NULL, resume, tf, rstr,
			    NULL);
   }
   fclose(f);
}

void resume_download_cb() {
   if (resume_dlg.window)
     return;
   create_resume_dlg();
   populate_resume_dlg();
}

void update_shares_cb() {
   shared_list_handle(user_info.upload_dir, UPDATE_LIST);   
}

FILE *open_servers(int status) {
   FILE *f;
   char *conf, *conf2;
   struct stat st;
   int x = -1;
   
   conf = g_strdup_printf("%s/.gnapster", getenv("HOME"));
   if (stat(conf, &st) < 0) {
      x = mkdir(conf, 0755);
      if (x < 0) {
	 perror("mkdir");
	 return NULL;
      }
      g_free(conf);
      conf2 = g_strdup_printf("%s/.gnapster_servers", getenv("HOME"));
      if (stat(conf2, &st) < 0) return NULL;
      conf = g_strdup_printf("%s/.gnapster/servers", getenv("HOME"));
      rename(conf2, conf);
      g_free(conf2);
   }
   g_free(conf);
   
   conf = g_strdup_printf("%s/.gnapster/servers", getenv("HOME"));
   f = fopen(conf, (status == 2) ? "a" : (status == 1) ? "w" : "r");
   if (!f) {
      perror("fopen");
      return NULL;
   }
   g_free(conf);
   
   return f;
}

/* The add/edit/delete routines are very hacky, group them together
 * later */
void add_dlg_cb(GtkWidget *w, char *data) {
   if (!data) return;
   
   if (!strcmp(data, "ok")) {
      FILE *f;
      char *server, *port, *desc;
      
      server = gtk_entry_get_text(GTK_ENTRY(add_dlg.server));
      port = gtk_entry_get_text(GTK_ENTRY(add_dlg.port));
      desc = gtk_entry_get_text(GTK_ENTRY(add_dlg.desc));
      if (!strlen(server) || !strlen(port) || !strlen(desc))
	return;

      f = open_servers(2);
      if (!f) {
	 gnome_error_dialog("Error opening servers file!");
	 return;
      }
      
      fprintf(f, "%s:%s %s\n", server, port, desc);

      fclose(f);
      
      gtk_clist_freeze(GTK_CLIST(browse_dlg.clist));
      gtk_clist_clear(GTK_CLIST(browse_dlg.clist));
      populate_browse_dlg_clist();
      gtk_clist_thaw(GTK_CLIST(browse_dlg.clist));
   }
   gnome_dialog_close(GNOME_DIALOG(add_dlg.window));
//   add_dlg.window = NULL;
}

void edit_dlg_cb(GtkWidget *w, NAP_SERVER *data) {
   if (data) {
      FILE *f;
      char buf[1024];
      char *cmp, *ptr, *temp;
      GList *entries = NULL, *listptr;
      char *server, *port, *desc;
      
      server = gtk_entry_get_text(GTK_ENTRY(edit_dlg.server));
      port = gtk_entry_get_text(GTK_ENTRY(edit_dlg.port));
      desc = gtk_entry_get_text(GTK_ENTRY(edit_dlg.desc));
      if (!strlen(server) || !strlen(port) || !strlen(desc))
	return;
      
      f = open_servers(0);
      if (!f) return;
      
      while(!feof(f)) {
	 fgets(buf, sizeof(buf) - 1, f);
	 if (feof(f)) break;
	 temp = strdup(buf);
	 ptr = next_arg(buf, NULL);
	 if (!ptr) break;
	 cmp = g_strdup_printf("%s:%i", data->ip, data->port);
	 if (strcmp(ptr, cmp))
	   entries = g_list_append(entries, temp);
	 else {
	    g_free(temp);
	    temp = g_strdup_printf("%s:%s %s\n", server, port,
				   desc);
	    entries = g_list_append(entries, temp);
	 }
	 g_free(cmp);
      }
      fclose(f);
      
      f = open_servers(1);
      listptr = entries;
      while(listptr) {
	 fprintf(f, "%s", (char *)listptr->data);
	 g_free(listptr->data);
	 listptr->data = NULL;
	 listptr = listptr->next;
      }
      fclose(f);
      g_list_free(entries);
      
      gtk_clist_freeze(GTK_CLIST(browse_dlg.clist));
      gtk_clist_clear(GTK_CLIST(browse_dlg.clist));
      populate_browse_dlg_clist();
      gtk_clist_thaw(GTK_CLIST(browse_dlg.clist));
   }
   gnome_dialog_close(GNOME_DIALOG(edit_dlg.window));
}

void browse_dlg_connect_cb(GtkWidget *w, int *data) {
   GtkCList *clist;
   NAP_SERVER *nsrv;
   
   if (conn_info.connected || conn_info.connecting) {
      gnome_error_dialog("You are already connected to a server!");
      return;
   }
   
   clist = GTK_CLIST(browse_dlg.clist);

   if (data)
     nsrv = gtk_clist_get_row_data(clist, *data);
   else
     nsrv = gnapster_get_selection_data(GTK_WIDGET(clist), NULL);
   if (!nsrv) return;
   
   DPRINTF(("ip = %s, port = %i\n", nsrv->ip, nsrv->port));
   
   g_free(conn_info.ip);
   conn_info.ip = strdup(nsrv->ip);
   conn_info.port = nsrv->port;
   conn_info.connecting = 1;
   conn_info.connected = 0;
   
   gnapster_update_status(_("Connecting to server..."), 1);
   gnome_config_set_string("/gnapster/Options/server", conn_info.ip);
   gnome_config_set_int("/gnapster/Options/server_port", conn_info.port);
   gnome_config_sync();
   napster_connect(conn_info.ip, conn_info.port);
   
   browse_dlg_cancel_cb();
}

void browse_dlg_cancel_cb() {
   gtk_clist_clear(GTK_CLIST(browse_dlg.clist));
   gnome_dialog_close(GNOME_DIALOG(browse_dlg.window));
   browse_dlg.window = NULL;
}

void browse_dlg_add() {
   create_add_dlg();
   gnome_dialog_run(GNOME_DIALOG(add_dlg.window));
}

void browse_dlg_edit() {
   NAP_SERVER *nsrv;
   int row = -1;
   char *desc;
   
   /* Basically just an add dialog but the callbacks are different... */
   nsrv = gnapster_get_selection_data(browse_dlg.clist, &row);
   if (!nsrv || row < 0) return;
   
   gtk_clist_get_text(GTK_CLIST(browse_dlg.clist), row, 0, &desc);
   if (!desc) return;
   if (!strlen(desc)) return;
   
   create_edit_dlg(nsrv, desc);
   gnome_dialog_run(GNOME_DIALOG(edit_dlg.window));
}

void browse_dlg_delete(GtkWidget *w, int *data) {
   FILE *f;
   char cmp[1024], buf[1024];
   char *ptr, *temp;
   NAP_SERVER *nsrv;
   GList *listptr, *entries = NULL;

   f = open_servers(0);
   if (!f) {
      gnome_error_dialog("Couldn't open servers file!");
      return;
   }

   if (data)
     nsrv = gtk_clist_get_row_data(GTK_CLIST(browse_dlg.clist), *data);
   else
     nsrv = gnapster_get_selection_data(browse_dlg.clist, NULL);
   if (!nsrv) return;

   while(!feof(f)) {
      fgets(buf, sizeof(buf) - 1, f);
      if (feof(f)) break;
      temp = strdup(buf);
      ptr = next_arg(buf, NULL);
      if (!ptr) break;
      
      g_snprintf(cmp, sizeof(cmp), "%s:%i",
		 nsrv->ip, nsrv->port);
      
      if (strcmp(ptr, cmp))
	entries = g_list_append(entries, temp);
   }
   fclose(f);
   
   f = open_servers(1);
   listptr = entries;
   while(listptr) {
      fprintf(f, "%s", (char *)listptr->data);
      g_free(listptr->data);
      listptr->data = NULL;
      listptr = listptr->next;
   }
   fclose(f);
   
   g_list_free(entries);

   if (!data) {
      gtk_clist_freeze(GTK_CLIST(browse_dlg.clist));
      gtk_clist_clear(GTK_CLIST(browse_dlg.clist));
      populate_browse_dlg_clist();
      gtk_clist_thaw(GTK_CLIST(browse_dlg.clist));
   }
}

void browse_dlg_button_cb(GtkWidget *widget, GdkEventButton *event, gpointer data) {
   int row = -1, col;

   gtk_clist_get_selection_info(GTK_CLIST(widget), event->x, event->y,
				&row, &col);

   if (event->type == GDK_2BUTTON_PRESS && event->button == 1) {
      browse_dlg_connect_cb(NULL, &row);
   }
}

void browse_dlg_refresh(GtkWidget *w, char *data) {
   
}

void populate_browse_dlg_clist() {
   char servers[1024];
   char buf[256], *ptr;
   char *add[6];
   FILE *f;
   NAP_SERVER *nsrv;

   g_snprintf(servers, sizeof(servers), "%s/.gnapster/servers",
	      getenv("HOME"));
   
   if (!file_exists(servers)) {
      f = open_servers(1);
      if (!f) return;
      /* Populate default server list */
      fprintf(f, "207.195.111.2:8888 bitchx.dimension6.com\n");
      fprintf(f, "205.161.144.22:8888 opennap.ownsu.net\n");
      fprintf(f, "216.207.184.105:8888 opennap.cx\n");
      fprintf(f, "206.161.89.187:8888 opennap.neglected.net\n");
      fclose(f);
   }
   
//   f = fopen(servers, "r");
   f = open_servers(0);
   if (!f) return;
   memset(add, 0, sizeof(add));
   while(!feof(f)) {
      fgets(buf, sizeof(buf) - 1, f);
      if (feof(f)) break;
      ptr = buf;

      add[1] = next_arg(ptr, &ptr);
      add[0] = last_arg(ptr, &ptr);
      
      if (add[0]) {
	 char *ptrbuf;

	 nsrv = g_malloc0(sizeof(NAP_SERVER));
	 ptrbuf = add[1];
	 if (!ptrbuf) {
	    g_free(nsrv);
	    continue;
	 }
	 nsrv->type = N_SERVER;
	 nsrv->ip = g_strdup(next_arg_full(ptrbuf, &ptrbuf, ':'));
	 nsrv->port = my_atoi(next_arg(ptrbuf, &ptrbuf));
	 nsrv->desc = g_strdup(add[0]);
	 
	 if (!nsrv->ip || nsrv->port <= 0) {
	    g_free(nsrv);
	    continue;
	 }
	 
	 gnapster_clist_append(browse_dlg.clist, NULL, nsrv, add[0],
			       add[1], "", "", "", NULL);
      }
   }
   fclose(f);
}

void browse_opennap_cb() {
   if (browse_dlg.window && !GTK_WIDGET_VISIBLE(browse_dlg.window)) return;
   create_browse_dlg();
}

void connect_official_cb() {
   gnome_config_set_string("/gnapster/Options/server", "208.184.216.222");
   gnome_config_set_int("/gnapster/Options/server_port", 8875);
   gnome_config_sync();
   if (conn_info.connected || conn_info.connecting) {
      gnome_error_dialog("You are already connected to a server!  Disconnect before attempting to use this operation");
      return;
   }
   connect_cb();
}

void switch_page_cb(GtkWidget *widget, GtkNotebookPage *page, int page_num,
		    gpointer data) {
   GtkWidget *nth_page, *label;
   GtkStyle *old_style;
   
   if (page_num == 4 || widget == console_tab.notebook)
     gtk_widget_grab_focus(console_tab.entry);
   
   nth_page = gtk_notebook_get_nth_page(GTK_NOTEBOOK(widget), page_num);
   label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(widget), nth_page);
   old_style = gtk_widget_get_style(label);
   if (old_style == gnapster_main.highlight)
     gtk_widget_set_style(label, gnapster_main.normal);
}

void build_list_cb() {
   if (building) {
      building = 0;
      return;
   }

   shared_list_handle(gtk_entry_get_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prop_dlg.upload_dir)))), 
		      BUILD_LIST);
   if (conn_info.connected)
     gnapster_text_insert(CONSOLE, "Please reconnect to the current server for the new mp3 list to take effect\n");
}

void update_list_cb() {
   shared_list_handle(gtk_entry_get_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prop_dlg.upload_dir)))),
		      UPDATE_LIST);
}

void list_chan_dlg_clist_cb(GtkWidget *widget, GdkEventButton *event, gpointer data) {
   int row = -1, col;
   
   gtk_clist_get_selection_info(GTK_CLIST(widget), event->x, event->y,
				&row, &col);
   
   if (event->type == GDK_2BUTTON_PRESS && event->button == 1) {
      list_chan_dlg_cb(NULL, "join");
   }
}

void list_chan_dlg_cb(GtkWidget *w, void *data) {
   if (!data) return;
   
   if (!strcmp(data, "join")) {
      NAP_CHANNEL *nchan;
      
      nchan = gnapster_get_selection_data(list_chan_dlg.clist, NULL);
      if (!nchan) return;
      napster_send(conn_info.sock, NAPSTER_JOIN_CHAN, nchan->chan);
   }
   
   gnome_dialog_close(GNOME_DIALOG(list_chan_dlg.window));
   list_chan_dlg.window = NULL;
}

void list_chan_cb(GtkWidget *w, gpointer data) {
   if (!conn_info.connected || conn_info.connecting) 
     return;
   create_list_chan_dlg();
   gtk_clist_freeze(GTK_CLIST(list_chan_dlg.clist));
   napster_send(conn_info.sock, NAPSTER_CHAN_LIST, NULL);
}

void join_chan_cb(GtkWidget *w, gpointer data) {
   gtk_notebook_set_page(GTK_NOTEBOOK(gnapster_main.notebook), 4);
   gtk_entry_set_text(GTK_ENTRY(console_tab.entry), "/join ");
   gtk_widget_grab_focus(console_tab.entry);
}

void part_chan_cb(GtkWidget *w, gpointer data) {
   if (!remove_console_tab(0))
     gnome_error_dialog("You must click on a channel tab before attempting to exit a channel");
}

void query_user_cb(GtkWidget *w, gpointer data) {
   gtk_notebook_set_page(GTK_NOTEBOOK(gnapster_main.notebook), 4);
   gtk_entry_set_text(GTK_ENTRY(console_tab.entry), "/query ");
   gtk_widget_grab_focus(console_tab.entry);
}

void unquery_user_cb(GtkWidget *w, gpointer data) {
   if (!remove_console_tab(1))
     gnome_error_dialog("You must click on a user query tab before attempting to close it");
}

void cddb_get() {
   CDDB_GET *temp;
   GtkWidget *clist = (GtkWidget *)cddb_tab.clist;
   GList *selection;
   
   selection = GTK_CLIST(clist)->selection;
   if (!selection) return;
   
   temp = gtk_clist_get_row_data(GTK_CLIST(clist), (int)selection->data);
   if (!temp) return;
   
   fprintf(stderr, "New GET: %s\n", temp->href);
}

void cddb_click_cb(GtkWidget *widget, GdkEventButton *event, gpointer data) {
   if (event->type == GDK_2BUTTON_PRESS && event->button == 1 &&
       widget == cddb_tab.clist) {
      cddb_get();
   }
}

void menu_popup_cb(GtkWidget *widget, GdkEventButton *event, gpointer data) {
   GtkWidget *temp_menu;
   int row = -1, col;

   gtk_clist_get_selection_info(GTK_CLIST(widget), event->x, event->y,
				&row, &col);

   if (event->type == GDK_2BUTTON_PRESS && event->button == 1 &&
       widget == search_tab.clist) {
      NAP_SEARCH *temp;
      NAP_DOWNLOAD *download;
      
      temp = gtk_clist_get_row_data(GTK_CLIST(widget), row);
      if (!temp) return;
      if (!temp->user) return;

      download = search_dup(temp, N_DOWNLOAD);
      handle_download(download, DOWNLOAD_NORMAL);
   } else if (event->button == 3) {
      temp_menu = gnome_popup_menu_new(data);
      gnome_popup_menu_do_popup_modal(temp_menu, NULL, NULL, event, widget);
      gtk_widget_destroy(temp_menu);
   }
}

void cddb_activate_cb() {
   if (!strlen(gtk_entry_get_text(GTK_ENTRY(cddb_tab.entry)))) return;

   gtk_clist_clear(GTK_CLIST(cddb_tab.clist));
   gtk_clist_freeze(GTK_CLIST(cddb_tab.clist));
   napster_connect_cddb();
}

void console_activate_cb() {
   char *console_text = NULL, *command, *message;
   
   console_text = gtk_entry_get_text(GTK_ENTRY(console_tab.entry));
   if (!console_text || !conn_info.connected) return;
   if (!strlen(console_text)) return;
   
   if (loc_end > MAX_HISTORY) {
      GList *link;
      
      link = g_list_nth(console_history, 0);
      console_history = g_list_remove_link(console_history, link);
      g_free(link->data);
      loc_end--;
   }
   console_history = g_list_append(console_history, strdup(console_text));
   loc_end++;
   loc_curr = loc_end;

   if (console_text[0] != '/') {
      do_sendtext(NULL, console_text);
      gtk_entry_set_text(GTK_ENTRY(console_tab.entry), "");
      return;
   }
   
   command = next_arg(console_text, &console_text);
   if (!command) {
      gtk_entry_set_text(GTK_ENTRY(console_tab.entry), "");
      return;
   }
   message = last_arg(console_text, &console_text);

   if (!handle_command(command, message)) {
      gnapster_text_insert(gnapster_get_pagenum(),
			   "* \3Error: \1unknown command (try \2/help\1)\n");
   }
   
   gtk_entry_set_text(GTK_ENTRY(console_tab.entry), "");
}

int console_keypress_cb(GtkWidget *w, GdkEventKey *key) {
   GList *ptr;

   if (key->keyval >= GDK_1 && key->keyval <= GDK_9 && 
       key->state & GDK_MOD1_MASK) {
      gtk_notebook_set_page(GTK_NOTEBOOK(console_tab.notebook),
			    (key->keyval - 49));
      return 1;
   }
   
   if (key->keyval != GDK_Up && key->keyval != GDK_Down)
     return 0;
   
   switch(key->keyval) {
    case GDK_Up:
      loc_curr--;
      if (loc_curr < 0) loc_curr = 0;
      break;
    case GDK_Down:
      loc_curr++;
      if (loc_curr > loc_end) loc_curr = loc_end;
      break;
   }
   ptr = g_list_nth(console_history, loc_curr);
   if (ptr) {
      char *text;
      
      text = ptr->data;
      if (text)
	gtk_entry_set_text(GTK_ENTRY(w), text);
   } else if (loc_curr == loc_end) {
      gtk_entry_set_text(GTK_ENTRY(w), "");
   }
	
   gtk_signal_emit_stop_by_name(GTK_OBJECT(w), "key_press_event");
   
   return 1;
}

NAP_SEARCH *get_data(const void *row) {
   return ((GtkCListRow *)row)->data;
}

int compare_file(GtkCList *clist, const void *row1, const void *row2) {
   NAP_SEARCH *s1 = get_data(row1);
   NAP_SEARCH *s2 = get_data(row2);
   
   return g_strcasecmp(s1->trunc_file, s2->trunc_file);
}

int compare_user(GtkCList *clist, const void *row1, const void *row2) {
   NAP_SEARCH *s1 = get_data(row1);
   NAP_SEARCH *s2 = get_data(row2);
   
   return g_strcasecmp(s1->user, s2->user);
}

int compare_size(GtkCList *clist, const void *row1, const void *row2) {
   NAP_SEARCH *s1 = get_data(row1);
   NAP_SEARCH *s2 = get_data(row2);
   
   if (s1->size <= s2->size) return 1;
   if (s1->size > s2->size) return -1;
   
   return 1;
}

int compare_length(GtkCList *clist, const void *row1, const void *row2) {
   NAP_SEARCH *s1 = get_data(row1);
   NAP_SEARCH *s2 = get_data(row2);
   int seconds1, seconds2;
   
   seconds1 = (s1->size / ((s1->kbps * 1000) / 8));
   seconds2 = (s2->size / ((s2->kbps * 1000) / 8));
   if (seconds1 <= seconds2) return 1;
   if (seconds1 > seconds2) return -1;
   
   return 1;
}

int compare_bitrate(GtkCList *clist, const void *row1, const void *row2) {
   NAP_SEARCH *s1 = get_data(row1);
   NAP_SEARCH *s2 = get_data(row2);
   
   if (s1->kbps <= s2->kbps) return 1;
   if (s1->kbps > s2->kbps) return -1;
   
   return 0;
}

int compare_connection(GtkCList *clist, const void *row1, const void *row2) {
   NAP_SEARCH *s1 = get_data(row1);
   NAP_SEARCH *s2 = get_data(row2);
   
   if (s1->connection <= s2->connection) return 1;
   if (s1->connection > s2->connection) return -1;
   
   return 0;
}

void search_sort(GtkCList *clist, int column) {
   switch(column) {
    case 0:
      gtk_clist_set_compare_func(clist, compare_file);
      break;
    case 1:
      gtk_clist_set_compare_func(clist, compare_connection);
      break;
    case 2:
      gtk_clist_set_compare_func(clist, compare_user);
      break;
    case 3:
      gtk_clist_set_compare_func(clist, compare_size);
      break;
    case 4:
      gtk_clist_set_compare_func(clist, compare_length);
      break;
    case 5:
      gtk_clist_set_compare_func(clist, compare_bitrate);
      break;
    default:
      gtk_clist_set_compare_func(clist, NULL);
   }
   
   if (clist->sort_column == column) {
      if (clist->sort_type == GTK_SORT_ASCENDING)
	clist->sort_type = GTK_SORT_DESCENDING;
      else
	clist->sort_type = GTK_SORT_ASCENDING;
   }
   
   gtk_clist_set_sort_column(clist, column);
   
   gtk_clist_sort(clist);
}

void search_activate_cb() {
   char *string, *media;
   char extra_optns[1024], temp[1024];
   
   if (!conn_info.connected || conn_info.searching) return;
   
   string = gtk_entry_get_text(GTK_ENTRY(search_tab.gtk_entry));
   if (!string || string[0] == 0) 
     return;
   string = strdup(string);

   gnapster_update_status(_("Searching..."), 1);
   conn_info.searching = 1;
   extra_optns[0] = '\0';
   if (user_info.conf[LINE_SPEED_SWITCH]) {
      char *cmp, *ptr;
      
      cmp = strdup(listint_to_str(user_info.conf[LINE_SPEED_CMP], 0));
      ptr = cmp;
      while(*ptr) {
	 *ptr = toupper(*ptr);
	 ptr++;
      }
      
      g_snprintf(temp, sizeof(temp), " LINESPEED \"%s\" %i", cmp, user_info.conf[LINE_SPEED]);
      strcat(extra_optns, temp);
      g_free(cmp);
   }
   if (user_info.conf[BITRATE_SWITCH]) {
      char *cmp, *ptr, *bit;
      
      cmp = strdup(listint_to_str(user_info.conf[BITRATE_CMP], 0));
      ptr = cmp;
      while(*ptr) {
	 *ptr = toupper(*ptr);
	 ptr++;
      }
      
      switch(user_info.conf[BITRATE]) {
       case 0:
	 bit = "96";
	 break;
       case 1:
	 bit = "112";
	 break;
       case 2:
	 bit = "128";
	 break;
       case 3:
	 bit = "160";
	 break;
       case 4:
	 bit = "192";
	 break;
       default:
	 bit = "";
	 break;
      }
      g_snprintf(temp, sizeof(temp), " BITRATE \"%s\" \"%s\"", cmp, bit);
      strcat(extra_optns, temp);
      g_free(cmp);
   }
   media = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(search_tab.media)->entry));
   if (g_strcasecmp(media, "audio") != 0) {
      g_snprintf(temp, sizeof(temp), " TYPE %s", media);
      strcat(extra_optns, temp);
   }
   g_strdown(string);
   napster_send(conn_info.sock, NAPSTER_SEARCH, "FILENAME CONTAINS \"%s\" MAX_RESULTS 100%s", 
		string, extra_optns);
   
   g_free(string);

   gtk_clist_clear(GTK_CLIST(search_tab.clist));
   gtk_clist_freeze(GTK_CLIST(search_tab.clist));
   gtk_entry_set_editable(GTK_ENTRY(search_tab.gtk_entry), 0);
}

void browse_user_menu(GtkWidget *clist) {
   void *temp;
   TYPE *n;
   int row = 0;
   char user[1024];
   
   if (!conn_info.connected) return;

   temp = gnapster_get_selection_data(clist, &row);
   if (!temp) return;
   
   n = (TYPE *)temp;
   
   if (n->type == N_SEARCH )
     g_snprintf(user, sizeof(user), ((NAP_SEARCH *)temp)->user);
   else if (n->type == N_DOWNLOAD)
     g_snprintf(user, sizeof(user), ((NAP_DOWNLOAD *)temp)->user);
   else if (n->type == N_USER)
     g_snprintf(user, sizeof(user), ((NAP_USER *)temp)->user);
   else 
     return;
   
   gnapster_update_status(_("Retrieving mp3 list..."), 1);
   conn_info.searching = 1;
   napster_send(conn_info.sock, NAPSTER_BROWSE, user);
   
   gtk_clist_clear(GTK_CLIST(browse_tab.ctree));
   gtk_clist_freeze(GTK_CLIST(browse_tab.ctree));
   gtk_entry_set_editable(GTK_ENTRY(browse_tab.entry), 0);
}

void browse_user_menu_cb(GtkWidget *w, gpointer data) {
   GtkWidget *data_widget;

   if (!data) return;

   data_widget = (GtkWidget *)data;
   browse_user_menu(data_widget);
}

void browse_user_cb() {
   char *string;
   
   if (GTK_CLIST(browse_tab.ctree)->freeze_count || !conn_info.connected ||
       conn_info.searching) 
     return;
   
   string = gtk_entry_get_text(GTK_ENTRY(browse_tab.entry));
   if (!strlen(string)) 
     return;
   
   gnapster_update_status(_("Retrieving mp3 list..."), 1);
   conn_info.searching = 1;
   napster_send(conn_info.sock, NAPSTER_BROWSE, string);
   
   gtk_clist_clear(GTK_CLIST(browse_tab.ctree));
   gtk_clist_freeze(GTK_CLIST(browse_tab.ctree));
   gtk_entry_set_editable(GTK_ENTRY(browse_tab.entry), 0);
}

void whois_user_menu_cb(GtkWidget *w, gpointer data) {
   GtkWidget *clist;
   int row = 0;
   NAP_SEARCH *temp;
   TYPE *n;
   char *user;
   
   if (!data) return;
   clist = (GtkWidget *)data;
   if (!clist) return;
   
   temp = gnapster_get_selection_data(clist, &row);
   if (!temp) return;
   
   n = (TYPE *)temp;
   
   if (n->type == N_SEARCH)
     user = temp->user;
   else if (n->type == N_USER) 
     user = ((NAP_USER *)temp)->user;
   else 
     return;

   gnapster_text_insert(gnapster_get_pagenum(), 
			"Sending whois request for \2%s\1...\n", user);
   napster_send(conn_info.sock, NAPSTER_WHOIS_REQUEST, user);
}

void connect_cb() {
   if (conn_info.connected || conn_info.connecting) return;
   if (!user_info.user) {
      gnapster_update_status(_("Please edit your preferences before attempting to connect!"), 0);
      return;
   }
   
   close(conn_info.sock);
   if (conn_info.sock_input) gdk_input_remove(conn_info.sock_input);
   conn_info.connected = 0;
   conn_info.connecting = 1;
   
   g_free(user_info.server);
   user_info.server = gnome_config_get_string("/gnapster/Options/server");
   user_info.server_port = gnome_config_get_int("/gnapster/Options/server_port");
   
   /* Napster servers changed their IP...this compensates... */
   if (!user_info.server || !strcmp(user_info.server, "208.184.216.223")) {
      gnome_config_set_string("/gnapster/Options/server", "208.184.216.222");
      gnome_config_set_int("/gnapster/Options/server_port", 8875);
      gnome_config_sync();
      user_info.server = gnome_config_get_string("/gnapster/Options/server");
      user_info.server_port = gnome_config_get_int("/gnapster/Options/server_port");
   }
   if (!user_info.server_port) {
      gnome_config_set_int("/gnapster/Options/server_port", 8875);
      gnome_config_sync();
      user_info.server_port = gnome_config_get_int("/gnapster/Options/server_port");
   }
   
   if (user_info.server_port == 8875) {
      /* Query server... */
      gnapster_update_status(_("Finding best host..."), 1);
      napster_get_best_host();
   } else {
      g_free(conn_info.ip);
      conn_info.ip = strdup(user_info.server);
      conn_info.port = user_info.server_port;
      conn_info.connecting = 1;
      conn_info.connected = 0;

      gnapster_update_status(_("Connecting to server..."), 1);
      napster_connect(conn_info.ip, conn_info.port);
   }
}

void disconnect_cb() {
   GList *ptr;
   
   if (!conn_info.connected && !conn_info.connecting) return;
   
   close(conn_info.sock);
   if (conn_info.sock_input) 
     gdk_input_remove(conn_info.sock_input);
   if (conn_info.upload_input) 
     gdk_input_remove(conn_info.upload_input);
   conn_info.connected = 0;
   conn_info.connecting = 0;
   conn_info.searching = 0;
   
   gtk_label_set_text(GTK_LABEL(upload_tab.label), "");
   gtk_entry_set_editable(GTK_ENTRY(search_tab.gtk_entry), 1);
   gtk_widget_set_sensitive(search_tab.media, 0);
   
   gnapster_update_status(_("Disconnected"), 0);
   gtk_text_set_point(GTK_TEXT(motd_tab.text), 0);
   gtk_text_forward_delete(GTK_TEXT(motd_tab.text), 
			   gtk_text_get_length(GTK_TEXT(motd_tab.text)));
   
   ptr = console_tab.channel_list;
   while(ptr) {
      if (((CHANNEL *)ptr->data)->clist)
	gtk_clist_clear(GTK_CLIST(((CHANNEL *)ptr->data)->clist));
      gtk_notebook_remove_page(GTK_NOTEBOOK(console_tab.notebook),
			       1);
      g_free(((CHANNEL *)ptr->data)->channel_name);
      g_free(ptr->data);
      ptr = ptr->next;
   }
   g_list_free(console_tab.channel_list);
   console_tab.channel_list = NULL;
}

void force_queue_cb(GtkWidget *w, gpointer data) {
   download_true_cb(download_tab.queue_clist, data);
}

void search_download_cb(GtkWidget *w, gpointer data) {
   download_true_cb(search_tab.clist, data);
}

void browse_download_cb(GtkWidget *w, gpointer data) {
   download_true_cb(browse_tab.ctree, data);
}

void download_true_cb(GtkWidget *w, gpointer data) {
   void *temp;
   GtkWidget *clist = w;
   GList *selection, *selptr = NULL;
   NAP_DOWNLOAD *download;
   
   selection = GTK_CLIST(clist)->selection;
   if (!selection) return;
   
//   selection = g_list_last(selection);
   
   while(selection) {
      if (w == browse_tab.ctree)
	temp = gtk_ctree_node_get_row_data(GTK_CTREE(clist), selection->data);
      else
	temp = gtk_clist_get_row_data(GTK_CLIST(clist), (int)selection->data);
      
      if (temp)
	selptr = g_list_append(selptr, temp);

      selection = selection->next;
   }
   
   selection = selptr;
   while(selection) {
      if (w == download_tab.queue_clist) {
	 temp = selection->data;
	 download = download_dup((NAP_DOWNLOAD *)temp);
	 handle_download(download, DOWNLOAD_FORCE);
	 gtk_clist_remove(GTK_CLIST(w),
			  gtk_clist_find_row_from_data(GTK_CLIST(w), (NAP_DOWNLOAD *)temp));
      } else {
	 temp = selection->data;
	 download = search_dup((NAP_SEARCH *)temp, N_DOWNLOAD);
	 handle_download(download, DOWNLOAD_NORMAL);
      }
      
      selection = selection->next;
   }
}

void cancel_upload_cb() {
   NAP_DOWNLOAD *temp;
   GtkWidget *clist = (GtkWidget *)upload_tab.clist;
   
   temp = gnapster_get_selection_data(clist, NULL);
   if (!temp) return;
   
   upload_close(temp, TRANSFER_CANCEL);
}

void cancel_download_cb() {
   NAP_DOWNLOAD *temp;
   GtkWidget *clist = (GtkWidget *)download_tab.clist;
   GList *selection, *selptr = NULL;

   selection = GTK_CLIST(clist)->selection;
   if (!selection) return;
   
   
   while(selection) {
      temp = gtk_clist_get_row_data(GTK_CLIST(clist), (int)selection->data);
      
      if (temp)
	selptr = g_list_append(selptr, temp);

      selection = selection->next;
   }
   
   selection = selptr;
   while(selection) {
      temp = selection->data;
      download_close(temp, TRANSFER_CANCEL);
      selection = selection->next;
   }
}

void cancel_remove_cb() {
   NAP_DOWNLOAD *temp;
   GtkWidget *clist = (GtkWidget *)download_tab.clist;
   GList *selection, *selptr = NULL;
   int l;

   selection = GTK_CLIST(clist)->selection;
   if (!selection) return;
   
   l = g_list_length(GTK_CLIST(clist)->row_list);
   
   while(selection) {
      if ((int)selection->data <= (l + 1)) {
	 temp = gtk_clist_get_row_data(GTK_CLIST(clist), (int)selection->data);
	 if (temp)
	   selptr = g_list_append(selptr, temp);
      }
      
      selection = selection->next;
   }
   
   selection = selptr;
   while(selection) {
      download_close((NAP_DOWNLOAD *)selection->data, 
		     TRANSFER_CANCEL | TRANSFER_FILE_REMOVE);
      
      selection = selection->next;
   }
}

void cancel_queue_cb() {
   NAP_DOWNLOAD *temp;
   GtkWidget *clist = (GtkWidget *)download_tab.queue_clist;
   GList *selection, *selptr = NULL;

   selection = GTK_CLIST(clist)->selection;
   if (!selection) 
     return; 

   while(selection) {
      temp = gtk_clist_get_row_data(GTK_CLIST(clist), (int)selection->data);
      if (temp)
	selptr = g_list_append(selptr, temp);

      selection = selection->next;
   }
   
   selection = selptr;
   while(selection) {
      temp = selection->data;
      gtk_clist_remove(GTK_CLIST(clist),
		       gtk_clist_find_row_from_data(GTK_CLIST(clist), temp));
      selection = selection->next;
   }
}

char *listint_to_str(int i, int type) {
   char *cmp[] = { 
      "At least", "Equal to", "At best", NULL
   }, *conn[] = {
      "Unknown", "14.4", "28.8", "33.6", "56k", "64k ISDN", "128k ISDN", "Cable", "DSL",
      "T1", "T3+", NULL
   };
   
   switch(type) {
    case 0:
      return cmp[i];
    case 1:
      return conn[i];
    default:
      return NULL;
   }
}

int liststr_to_int(char *s) {
   char *conn[] = {
      "Unknown", "14.4", "28.8", "33.6", "56k", "64k ISDN", "128k ISDN", "Cable", "DSL",
      "T1", "T3+", NULL
   };
   char *bitrate[] = {
      "96", "112", "128", "160", "192", NULL
   };
   int i = 0;

   if (!strcmp(s, "At least"))
     return 0;
   else if (!strcmp(s, "Equal to"))
     return 1;
   else if (!strcmp(s, "At best"))
     return 2;
   
   while(conn[i]) {
      if (!strcmp(s, conn[i])) return i;
      i++;
   }
   i = 0;
   while(bitrate[i]) {
      if (!strcmp(s, bitrate[i])) return i;
      i++;
   }

   return 0;
}

void combo_changed(GtkWidget *widget, gpointer data) {
   WidgetProperty *prop;
   int val;
   
   prop = gtk_object_get_data(GTK_OBJECT(widget), "prop");
   if (!prop)
     return;
   
   if (GTK_IS_SPIN_BUTTON(widget))
     val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   else
     val = liststr_to_int(gtk_entry_get_text(GTK_ENTRY(widget)));
   user_info.conf[prop->key] = val;

   gnome_config_set_int(prop->conf, val);
   gnome_config_sync();
}

void preferences_changed_cb() {
   if (!prop_dlg.window) return;
   gnome_property_box_changed(GNOME_PROPERTY_BOX(prop_dlg.window));
}

void preferences_button_cb(GtkWidget *widget, gint arg1, gpointer data) {
   char *user, *pass, *nap_port, *conn, *download_dir, *upload_dir;

   prop_dlg.calling++;
   if (prop_dlg.calling == 2) {
      prop_dlg.calling = 0;
      return;
   }
   
   user = gtk_entry_get_text(GTK_ENTRY(prop_dlg.user_entry));
   pass = gtk_entry_get_text(GTK_ENTRY(prop_dlg.pass_entry));
   nap_port = gtk_entry_get_text(GTK_ENTRY(prop_dlg.nap_port));
   conn = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(prop_dlg.connection)->entry));
   download_dir = gtk_entry_get_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prop_dlg.download_dir))));
   upload_dir = gtk_entry_get_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prop_dlg.upload_dir))));
   
//   server = gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(prop_dlg.server_entry))));

   if (!strlen(user) || !strlen(pass) || !strlen(conn) || !strlen(download_dir) || !strlen(nap_port)) {
      gnapster_update_status(_("Form not complete!  Make sure you fill in everything!"), 0);
      return;
   }
   
   user_info.connection = napster_conn_as_int(conn);
   if (my_atoi(nap_port) != conn_info.nap_port || user_info.conf[FIREWALL] == 0) {
      conn_info.nap_port = my_atoi(nap_port);
      gdk_input_remove(conn_info.bind_sock_input);
      close(conn_info.bind_sock);
      if (conn_info.nap_port)
	napster_bind(conn_info.nap_port);
      if (conn_info.connected) {
	 napster_send(conn_info.sock, NAPSTER_CHANGE_PORT, "%i",
		      conn_info.nap_port);
      }
      gnome_config_set_int("/gnapster/Options/port", conn_info.nap_port);
   } else if (user_info.conf[FIREWALL] == 1) {
      conn_info.nap_port = 0;
      gdk_input_remove(conn_info.bind_sock_input);
      close(conn_info.bind_sock);
      if (conn_info.connected)
	napster_send(conn_info.sock, NAPSTER_CHANGE_PORT, "0");
   }
   
   gnome_config_set_string("/gnapster/User/user", user);
   gnome_config_set_string("/gnapster/User/pass", pass);
   gnome_config_set_int("/gnapster/User/connection", user_info.connection);
   gnome_config_set_string("/gnapster/User/dl_dir", download_dir);
   if (strlen(upload_dir)) 
     gnome_config_set_string("/gnapster/User/ul_dir", upload_dir);
   
   gnapster_config_set_int(0);
   
   g_free(user_info.user);
   g_free(user_info.pass);
   g_free(user_info.download_dir);
   g_free(user_info.upload_dir);
   user_info.user = gnome_config_get_string("/gnapster/User/user");
   user_info.pass = gnome_config_get_string("/gnapster/User/pass");
   user_info.download_dir = gnome_config_get_string("/gnapster/User/dl_dir");   
   user_info.upload_dir = gnome_config_get_string("/gnapster/User/ul_dir");
   
   if (!conn_info.connected && !conn_info.connecting)
     gnapster_update_status(_("Not connected (use the file menu to connect)"), -1);
}

void preferences_destroy_cb(GtkWidget *widget, gint arg1, gpointer data) {
   prop_dlg.window = NULL;
}

void clicked_cb(GtkWidget *widget, int *data) {
   WidgetProperty *prop;
   GList *ptr;
   
   *data = !*data;
   
   prop = gtk_object_get_data(GTK_OBJECT(widget), "prop");
   if (!prop)
     return;
   
   gnome_config_set_int(prop->conf, *data);
   ptr = prop->sensitive;
   while(ptr) {
      gtk_widget_set_sensitive(GTK_WIDGET(ptr->data), *data);
      ptr = ptr->next;
   }
   
/*   if (widget == search_tab.line_speed_switch) {
      gnome_config_set_int("/gnapster/Options/line_speed_switch", *data);
      gtk_widget_set_sensitive(search_tab.line_speed_cmp, *data);
      gtk_widget_set_sensitive(search_tab.line_speed, *data);
   } else if (widget == search_tab.bitrate_switch) {
      gnome_config_set_int("/gnapster/Options/bitrate_switch", *data);
      gtk_widget_set_sensitive(search_tab.bitrate_cmp, *data);
      gtk_widget_set_sensitive(search_tab.bitrate, *data);
   } else if (widget == upload_tab.max_uploads_switch) {
      gnome_config_set_int("/gnapster/Options/max_uploads_switch", *data);
      gtk_widget_set_sensitive(upload_tab.max_uploads, *data);
   } else if (widget == upload_tab.max_peruser_uploads_switch) {
      gnome_config_set_int("/gnapster/Options/max_peruser_uploads_switch", *data);
      gtk_widget_set_sensitive(upload_tab.max_peruser_uploads, *data);
   } else if (widget == download_tab.max_downloads_switch) {
      gnome_config_set_int("/gnapster/Options/max_downloads_switch", *data);
      gtk_widget_set_sensitive(download_tab.max_downloads, *data);
   } else
     return;*/
   
   gnome_config_sync();
}

void preferences_clicked_cb(GtkWidget *widget, int *data) {
   clicked_cb(widget, data);
   preferences_changed_cb();
}

void preferences_cb() {
   if (prop_dlg.window) return;
   
   create_prop_dlg(preferences_changed_cb,
		   preferences_button_cb,
		   preferences_destroy_cb,
		   preferences_clicked_cb,
		   &(user_info.new_user));
   gtk_widget_show_all(prop_dlg.window);
}

void build_dlg_cancel_cb() {
   building = 0;
   gnome_dialog_close(GNOME_DIALOG(build_dlg.window));
   gdk_input_remove(build_dlg.pipe_input);
   if (build_dlg.cpid)
     kill(build_dlg.cpid, SIGUSR1);
}

void about_cb() {
   GtkWidget *about_box;
   const char *authors[] = { "jasta <jasta@gotlinux.org>", NULL };
   
   about_box = gnome_about_new(_("Gnapster"), VERSION,
			       _("Copyright (C) 1999-2000 Josh Guilfoyle"),
			       authors,
			       _("Napster (online mp3 community) client written for GNOME"),
			       NULL);
   gtk_widget_show(about_box);
}
